@@ -27,13 +27,15 @@ struct idle_prof_thread {
 	int state;
 	struct timeval tps;
 	struct timeval tpe;
-	double cali_time; /* microseconds to finish a unit wrok */
+	double cali_time; /* microseconds to finish a unit work */
 	double loops;
 	double idleness;
 	unsigned char *data;             /* bytes to be touched */
 	pthread_cond_t  cond;
 	pthread_mutex_t init_lock;
 	pthread_mutex_t start_lock;
+
+	os_cpu_mask_t cpu_mask;
 };
 
 struct idle_prof_common {
